@@ -7,13 +7,94 @@ import statsmodels.api as sm
 import warnings
 
 
-__all__ = ['lowpass', 'calibrate_gravity', 'detect_nonwear', 'resample', 'get_stationary_indicator']
+__all__ = ['quality_control', 'lowpass', 'calibrate_gravity', 'flag_nonwear', 'find_nonwear_segments', 'resample']
+
+
+def quality_control(data, sample_rate):
+    """
+    Perform basic quality control on the provided data.
+
+    This function performs the following tasks:
+    1. Returns a dictionary with general information about the data.
+    2. Checks for non-increasing timestamps and corrects them if necessary, returning the corrected data.
+
+    :param data: A pandas.DataFrame of acceleration time-series. The index must be a DateTimeIndex.
+    :type data: pandas.DataFrame
+    :param sample_rate: Target sample rate (Hz) to achieve.
+    :type sample_rate: int or float
+    :return: A tuple containing the processed data and a dictionary with general information about the data.
+        The dictionary contains the following:
+
+        - **NumTicks**: Total number of ticks (samples) in the data.
+        - **StartTime**: First timestamp of the data.
+        - **EndTime**: Last timestamp of the data.
+        - **WearTime(days)**: Total wear time, in days. This is simply the total \
+            duration of valid (non-NaN) data and does not account for potential \
+            nonwear segments. See ``find_nonwear_segments`` and ``flag_nonwear`` to \
+            find and flag nonwear segments in the data.
+        - **DataSpan(days)**: Time span of the data (difference between last and first timestamps).
+        - **NumInterrupts**: The number of interruptions in the data (gaps or NaNs between samples).
+        - **ReadErrors**: The number of data errors (if non-increasing timestamps are found).
+        - **Covers24hOK**: Whether the data covers all 24 hours of the day.
+    :rtype: (pandas.DataFrame, dict)
+    """
+
+    info = {}
+
+    if len(data) == 0:
+        info['ReadErrors'] = 0
+        info['StartTime'] = None
+        info['EndTime'] = None
+        info['NumTicks'] = 0
+        info['WearTime(days)'] = 0
+        info['DataSpan(days)'] = 0
+        info['NumInterrupts'] = 0
+        return data, info
+
+    # Check for non-increasing timestamps. This is rare but can happen with
+    # buggy devices. TODO: Parser should do this.
+    tdiff = data.index.to_series().diff()  # Note: Index.diff() was only added in pandas 2.1
+    errs = (tdiff <= pd.Timedelta(0)).sum()
+    del tdiff  # we're done with this
+    if errs > 0:
+        print("Found non-increasing data timestamps. Fixing...")
+        data = data[data
+                    .index
+                    .to_series()
+                    .cummax()
+                    .diff()
+                    .fillna(pd.Timedelta(1))
+                    .gt(pd.Timedelta(0))]
+        info['ReadErrors'] = int(np.ceil(errs / sample_rate))
+    else:
+        info['ReadErrors'] = 0
+
+    # Start/end times, wear time, interrupts
+    time_format = "%Y-%m-%d %H:%M:%S"
+    info['StartTime'] = data.index[0].strftime(time_format)
+    info['EndTime'] = data.index[-1].strftime(time_format)
+    info['NumTicks'] = len(data)
+    tol = pd.Timedelta('1s')
+    tdiff = data.dropna().index.to_series().diff()  # Note: Index.diff() was only added in pandas 2.1
+    total_time = tdiff[tdiff < tol].sum().total_seconds()
+    num_interrupts = (tdiff > tol).sum()
+    del tdiff  # we're done with this
+    info['WearTime(days)'] = total_time / (60 * 60 * 24)
+    info['DataSpan(days)'] = (data.index[-1] - data.index[0]).total_seconds() / (60 * 60 * 24)
+    info['NumInterrupts'] = num_interrupts
+
+    # Check if data covers all 24 hours of the day
+    coverage = data.notna().any(axis=1).groupby(data.index.hour).mean()
+    info['Covers24hOK'] = int(len(coverage) == 24 and np.min(coverage) >= 0.01)
+    del coverage
+
+    return data, info
 
 
 def resample(data, sample_rate, dropna=False, chunksize=1_000_000):
     """
     Nearest neighbor resampling. For downsampling, it is recommended to first
-    apply an antialiasing filter.
+    apply an antialiasing filter (e.g. a low-pass filter, see ``lowpass``).
 
     :param data: A pandas.DataFrame of acceleration time-series. The index must be a DateTimeIndex.
     :type data: pandas.DataFrame.
@@ -165,13 +246,13 @@ def lowpass(data, data_sample_rate, cutoff_rate=20, chunksize=1_000_000):
     return data, info
 
 
-def detect_nonwear(data, patience='90m', window='10s', stdtol=15 / 1000):
+def flag_nonwear(data, patience='90m', window='10s', stdtol=15 / 1000):
     """
-    Detect nonwear episodes based on long periods of no movement.
+    Flag nonwear episodes in the data by setting them to NA. Non-wear episodes are inferred from long periods of no movement.
 
     :param pandas.DataFrame data: A pandas.DataFrame of acceleration time-series. The index must be a DateTimeIndex.
     :type data: pandas.DataFrame.
-    :param patience: Minimum length of the stationary period to be flagged as non-wear. Defaults to 90 minutes ("90m").
+    :param patience: The minimum duration that a stationary episode must have to be classified as non-wear episode. Defaults to 90 minutes ("90m").
     :type patience: str, optional
     :param window: Rolling window to use to check for stationary periods. Defaults to 10 seconds ("10s").
     :type window: str, optional
@@ -183,48 +264,35 @@ def detect_nonwear(data, patience='90m', window='10s', stdtol=15 / 1000):
 
     info = {}
 
-    stationary_indicator = (  # this is more memory friendly than data[['x', 'y', 'z']].std()
-        data['x'].resample(window, origin='start').std().lt(stdtol)
-        & data['y'].resample(window, origin='start').std().lt(stdtol)
-        & data['z'].resample(window, origin='start').std().lt(stdtol)
-    )
+    nonwear_segments = find_nonwear_segments(data, patience=patience, window=window, stdtol=stdtol)
 
-    segment_edges = (stationary_indicator != stationary_indicator.shift(1))
-    segment_edges.iloc[0] = True  # first edge is always True
-    segment_ids = segment_edges.cumsum()
-    stationary_segment_ids = segment_ids[stationary_indicator]
-    stationary_segment_lengths = (
-        stationary_segment_ids
-        .groupby(stationary_segment_ids)
-        .agg(
-            start_time=lambda x: x.index[0],
-            length=lambda x: x.index[-1] - x.index[0]
-        )
-        .set_index('start_time')
-        .squeeze(axis=1)
-        # dtype defaults to int64 when series is empty, so
-        # astype('timedelta64[ns]') makes sure it's always a timedelta,
-        # otherwise comparison with Timedelta(patience) below will fail
-        .astype('timedelta64[ns]')
-    )
-    nonwear_segment_lengths = stationary_segment_lengths[stationary_segment_lengths > pd.Timedelta(patience)]
-
-    count_nonwear = len(nonwear_segment_lengths)
-    total_nonwear = nonwear_segment_lengths.sum().total_seconds()
-    total_wear = (
-        data.index.to_series().diff()
-        .pipe(lambda x: x[x < pd.Timedelta('1s')].sum())
-        .total_seconds()
-    ) - total_nonwear
-
-    info['WearTime(days)'] = total_wear / (60 * 60 * 24)
-    info['NonwearTime(days)'] = total_nonwear / (60 * 60 * 24)
-    info['NumNonwearEpisodes'] = count_nonwear
+    # Num nonwear episodes and total nonwear time
+    count_nonwear = len(nonwear_segments)
+    total_nonwear = nonwear_segments.sum().total_seconds()
 
     # Flag nonwear segments
     data = data.copy(deep=True)  # copy to avoid modifying original data
-    for start_time, length in nonwear_segment_lengths.items():
-        data.loc[start_time:start_time + length] = np.nan
+    for start_time, length in nonwear_segments.items():
+        data.loc[start_time : start_time + length] = np.nan
+    del nonwear_segments
+
+    # Calculate total wear time and interrupts (data gaps)
+    tol = pd.Timedelta('1s')
+    tdiff = data.dropna().index.to_series().diff()  # Note: Index.diff() was only added in pandas 2.1
+    total_time = tdiff[tdiff < tol].sum().total_seconds()
+    num_interrupts = (tdiff > tol).sum()
+    del tdiff
+
+    # Check if data covers all 24 hours of the day
+    coverage = data.notna().any(axis=1).groupby(data.index.hour).mean()
+    covers24hok = int(len(coverage) == 24 and np.min(coverage) >= 0.01)
+    del coverage
+
+    info['NonwearTime(days)'] = total_nonwear / (60 * 60 * 24)
+    info['NumNonwearEpisodes'] = count_nonwear
+    info['WearTime(days)'] = total_time / (60 * 60 * 24)
+    info['NumInterrupts'] = num_interrupts
+    info['Covers24hOK'] = covers24hok
 
     return data, info
 
@@ -448,41 +516,88 @@ def calibrate_gravity(data, calib_cube=0.3, calib_min_samples=50, window='10s',
     return data, info
 
 
-def get_stationary_indicator(data, window='10s', stdtol=15 / 1000):
-    """
-    Return a boolean pandas.Series indicating stationary (low movement) periods.
+# def get_stationary_indicator(data, window='10s', stdtol=15 / 1000):
+#     """
+#     Return a boolean pandas.Series indicating stationary (low movement) periods.
 
-    :param data: A pandas.DataFrame of acceleration time-series. It must contain
-        at least columns `x,y,z` and the index must be a DateTimeIndex.
+#     :param data: A pandas.DataFrame of acceleration time-series. It must contain
+#         at least columns `x,y,z` and the index must be a DateTimeIndex.
+#     :type data: pandas.DataFrame.
+#     :param window: Rolling window to use to check for stationary periods. Defaults to 10 seconds ("10s").
+#     :type window: str, optional
+#     :param stdtol: Standard deviation under which the window is considered stationary.
+#         Defaults to 15 milligravity (0.015).
+#     :type stdtol: float, optional
+#     :return: Boolean pandas.Series indexed as `data` indicating stationary periods.
+#     :rtype: pandas.Series
+#     """
+
+#     def fn(data):
+#         return (
+#             (data[['x', 'y', 'z']]
+#              .rolling(window)
+#              .std()
+#              < stdtol)
+#             .all(axis=1)
+#         )
+
+#     stationary_indicator = pd.concat(
+#         chunker(
+#             data,
+#             chunksize='4h',
+#             leeway=window,
+#             fn=fn
+#         )
+#     )
+
+#     return stationary_indicator
+
+
+def find_nonwear_segments(data, patience='90m', window='10s', stdtol=15 / 1000):
+    """
+    Find nonwear episodes based on long periods of no movement.
+
+    :param pandas.DataFrame data: A pandas.DataFrame of acceleration time-series. The index must be a DateTimeIndex.
     :type data: pandas.DataFrame.
+    :param patience: The minimum duration that a stationary episode must have to be classified as non-wear episode. Defaults to 90 minutes ("90m").
+    :type patience: str, optional
     :param window: Rolling window to use to check for stationary periods. Defaults to 10 seconds ("10s").
     :type window: str, optional
-    :param stdtol: Standard deviation under which the window is considered stationary.
-        Defaults to 15 milligravity (0.015).
+    :param stdtol: Standard deviation under which the window is considered stationary. Defaults to 15 milligravity (0.015).
     :type stdtol: float, optional
-    :return: Boolean pandas.Series indexed as `data` indicating stationary periods.
+    :return: A Series where the DatetimeIndex indicates the start times of each non-wear segment and the values are the length
+        of each segment, in timedelta64[ns].
     :rtype: pandas.Series
     """
 
-    def fn(data):
-        return (
-            (data[['x', 'y', 'z']]
-             .rolling(window)
-             .std()
-             < stdtol)
-            .all(axis=1)
-        )
-
-    stationary_indicator = pd.concat(
-        chunker(
-            data,
-            chunksize='4h',
-            leeway=window,
-            fn=fn
-        )
+    stationary_indicator = (  # this is more memory friendly than data[['x', 'y', 'z']].std()
+        # use ffill() so that NA also counted as stationary
+        data['x'].ffill().resample(window, origin='start').std().lt(stdtol)
+        & data['y'].ffill().resample(window, origin='start').std().lt(stdtol)
+        & data['z'].ffill().resample(window, origin='start').std().lt(stdtol)
     )
 
-    return stationary_indicator
+    segment_edges = (stationary_indicator != stationary_indicator.shift(1))
+    segment_edges.iloc[0] = True  # first edge is always True
+    segment_ids = segment_edges.cumsum()
+    stationary_segment_ids = segment_ids[stationary_indicator]
+    stationary_segment_lengths = (
+        stationary_segment_ids
+        .groupby(stationary_segment_ids)
+        .agg(
+            start_time=lambda x: x.index[0],
+            length=lambda x: x.index[-1] - x.index[0]
+        )
+        .set_index('start_time')
+        .squeeze(axis=1)
+        # dtype defaults to int64 when series is empty, so
+        # astype('timedelta64[ns]') makes sure it's always a timedelta,
+        # otherwise comparison with Timedelta(patience) below will fail
+        .astype('timedelta64[ns]')
+    )
+    nonwear_segment_lengths = stationary_segment_lengths[stationary_segment_lengths > pd.Timedelta(patience)]
+
+    return nonwear_segment_lengths
 
 
 def get_wear_time(t, tol=0.1):
