@@ -21,6 +21,10 @@ def read_device(input_file,
                 calibrate_gravity=True,
                 detect_nonwear=True,
                 resample_hz='uniform',
+                start_time=None,
+                end_time=None,
+                skipdays=0,
+                cutdays=0,
                 verbose=True):
     """
     Read and process accelerometer device file. Returns a pandas.DataFrame with
@@ -39,6 +43,16 @@ def read_device(input_file,
         "uniform", use the implied frequency (use this option to fix any device
         sampling errors). Pass None to disable. Defaults to "uniform".
     :type resample_hz: str or int, optional
+    :param start_time: Start time to read data. Pass None to read from the
+        beginning. Defaults to None.
+    :type start_time: str or datetime, optional
+    :param end_time: End time to read data. Pass None to read until the end.
+        Defaults to None.
+    :type end_time: str or datetime, optional
+    :param skipdays: Number of days to skip from the beginning. Defaults to 0.
+    :type skipdays: int, optional
+    :param cutdays: Number of days to cut from the end. Defaults to 0.
+    :type cutdays: int, optional
     :param verbose: Verbosity, defaults to True.
     :type verbose: bool, optional
     :return: Processed data and processing info.
@@ -49,6 +63,18 @@ def read_device(input_file,
 
     data, info = _read_device(input_file, verbose)
 
+    # Filter data by start/end time, if specified
+    if start_time is not None:
+        data = data.loc[start_time:]
+    if end_time is not None:
+        data = data.loc[:end_time]
+
+    # Skip/cut days, if specified
+    if skipdays > 0:
+        data = data.loc[data.index[0] + pd.Timedelta(days=skipdays):]
+    if cutdays > 0:
+        data = data.loc[:data.index[-1] - pd.Timedelta(days=cutdays)]
+
     # data, info_process = process(data, info_read['SampleRate'],
     #                              lowpass_hz=lowpass_hz,
     #                              calibrate_gravity=calibrate_gravity,
@@ -61,7 +87,7 @@ def read_device(input_file,
 
     # Basic quality control
     timer.start("Quality control...")
-    data, info_qc = qc(data, info['SampleRate'])
+    data, info_qc = P.quality_control(data, info['SampleRate'])
     info_qc['ReadErrors'] += info['ReadErrors']
     info.update(info_qc)
     timer.stop()
@@ -84,7 +110,7 @@ def read_device(input_file,
 
     if detect_nonwear:
         timer.start("Nonwear detection...")
-        data, info_nonwear = P.detect_nonwear(data)
+        data, info_nonwear = P.flag_nonwear(data)
         info.update(info_nonwear)
         timer.stop()
 
@@ -150,7 +176,7 @@ def process(data, sample_rate,
 
     if detect_nonwear:
         timer.start("Nonwear detection...")
-        data, info_nonwear = P.detect_nonwear(data)
+        data, info_nonwear = P.flag_nonwear(data)
         info.update(info_nonwear)
         timer.stop()
 
@@ -166,59 +192,6 @@ def process(data, sample_rate,
     return data, info
 
 
-def qc(data, sample_rate):
-    """ Basic quality control. Returns a dict with general info. Also checks and
-    corrects for non-increasing timestamps.
-    """
-
-    info = {}
-
-    if len(data) == 0:
-        info['NumTicks'] = 0
-        info['StartTime'] = None
-        info['EndTime'] = None
-        info['WearTime(days)'] = 0
-        info['NumInterrupts'] = 0
-        info['ReadErrors'] = 0
-        return info
-
-    # Start/end times, wear time, interrupts
-    tol = pd.Timedelta('1s')
-    tdiff = data.index.to_series().diff()  # Note: Index.diff() was only added in pandas 2.1
-    total_wear = tdiff[tdiff < tol].sum().total_seconds()
-    num_interrupts = (tdiff > tol).sum()
-    time_format = "%Y-%m-%d %H:%M:%S"
-    info['NumTicks'] = len(data)
-    info['StartTime'] = data.index[0].strftime(time_format)
-    info['EndTime'] = data.index[-1].strftime(time_format)
-    info['WearTime(days)'] = total_wear / (60 * 60 * 24)
-    info['NumInterrupts'] = num_interrupts
-
-    # Check for non-increasing timestamps. This is rare but can happen with
-    # buggy devices. TODO: Parser should do this.
-    errs = (tdiff <= pd.Timedelta(0)).sum()
-    del tdiff  # we're done with this
-    if errs > 0:
-        print("Found non-increasing data timestamps. Fixing...")
-        data = data[data
-                    .index
-                    .to_series()
-                    .cummax()
-                    .diff()
-                    .fillna(pd.Timedelta(1))
-                    .gt(pd.Timedelta(0))]
-        info['ReadErrors'] = int(np.ceil(errs / sample_rate))
-    else:
-        info['ReadErrors'] = 0
-
-    # Check if data covers all 24 hours of the day
-    coverage = data.notna().any(axis=1).groupby(data.index.hour).mean()
-    info['Covers24hOK'] = int(len(coverage) == 24 and np.min(coverage) >= 0.01)
-    del coverage
-
-    return data, info
-
-
 def _read_device(input_file, verbose=True):
     """ Internal function that interfaces with the Java parser to read the
     device file. Returns parsed data as a pandas dataframe, and a dict with
@@ -252,14 +225,15 @@ def _read_device(input_file, verbose=True):
         timer.stop()
 
         timer.start("Converting to dataframe...")
-        # NOTE: Care is taken to avoid excessive memory usage. First we open the
-        # file as mmap, then we load each column using np.asarray (np.array uses
-        # more memory - not sure why), and finally we convert to
-        # pandas.DataFrame with copy=False to avoid re-copying.
+        # NOTE: To reduce memory usage, we open the file as mmap and use
+        # np.asarray to define the dict for the dataframe construction. Note
+        # that the dataframe uses copy=True because we need to remove all
+        # references to the mmap object so that the temporary directory can be
+        # deleted later (another way is to use np.array and copy=False).
         data_mmap = np.load(os.path.join(tmpdir, "data.npy"), mmap_mode='r')
-        data = {c: np.asarray(data_mmap[c]) for c in data_mmap.dtype.names}
-        data = pd.DataFrame(data, copy=False)
+        data = pd.DataFrame({c: np.asarray(data_mmap[c]) for c in data_mmap.dtype.names}, copy=True)
         data.set_index('time', inplace=True)
+        del data_mmap  # delete this so that the temporary directory can be deleted
         timer.stop()
 
         return data, info
@@ -268,9 +242,13 @@ def _read_device(input_file, verbose=True):
 
         # Cleanup, delete temporary directory
         try:
+            # NOTE: For the tmpdir to be deleted, all references to the mmap
+            # object must have been deleted. This includes data_mmap, but also
+            # indirect references like the dataframe (copy=False) or arrays
+            # created with np.asarray.
             shutil.rmtree(tmpdir)
         except OSError as e:
-            print("Error: %s - %s." % (e.filename, e.strerror))
+            print(f"Error: {e.filename} - {e.strerror}.")
 
 
 def java_read_device(input_file, output_dir, verbose=True):
